function Decoder(audioCtx){
this.audioCtx = audioCtx;
this.Fs = this.audioCtx.sampleRate;
var firCalculator =  new Fili.FirCoeffs();
var IIRCascadeCalculator = new Fili.CalcCascades();
var firFilterCoeffs = firCalculator.lowpass({
   								 order: 250, // filter order
    							 Fs: this.Fs, // sampling frequency
    							 Fc: 3000 // cutoff frequency
 								 });
var lowShelfCoeffs = IIRCascadeCalculator.lowshelf({
	order: 3,
	characteristic: "butterworth",
	Fs: this.Fs,
	Fc: 3200,
	gain: 3,
});
this.lowShelfFilter = new Fili.IirFilter(lowShelfCoeffs);
this.firLpf = new Fili.FirFilter(firFilterCoeffs);
lowShelfCoeffs = null;
firFilterCoeffs = null;
IIRCascadeCalculator = null;
firCalculator = null;
this.demodulator;
this.osc;
this.demodulatedBuffer = [];
this.inter;
this.endTime = 0;
}



Decoder.prototype.init = function(bufferSrc, samples){
	this.notchFilter = this.audioCtx.createBiquadFilter();
	this.notchFilter.type = 'notch';
	this.notchFilter.Q.value = 20;
	this.notchFilter.frequency.value = 6900;//notch at 6900Hz

	this.osc =  this.audioCtx.createOscillator();
	this.osc.type = 'sine';
	this.osc.frequency.value = 6500;
	var a_buffer = this.audioCtx.createBuffer(1, samples.length, this.Fs);
	a_buffer.getChannelData(0).set(samples);
	bufferSrc.buffer = a_buffer;
	this.merger = this.audioCtx.createChannelMerger(2);
	this.demodulator = (this.audioCtx.createScriptProcessor || this.audioCtx.createJavaScriptNode).call(this.audioCtx, 8192, 2, 1);
	this.demodulator.onaudioprocess = this.process.bind(this);
	bufferSrc.onended = this.finish.bind(this);
	this.osc.connect(this.merger,0,0);
	//bufferSrc.connect(this.notchFilter).connect(this.merger,0,1);
	bufferSrc.connect(this.merger,0,1);
	this.merger.connect(this.demodulator);
	bufferSrc.playbackRate.value = 10;//10 times faster;
	this.demodulator.connect(this.audioCtx.destination);
	this.osc.start();
	bufferSrc.start();
};

Decoder.prototype.returnBlobWav = function(){};
Decoder.prototype.demodError = function(){};
Decoder.prototype.decodeProcess = function(){};
Decoder.prototype.decodeProcessEnd = function(){};

Decoder.prototype.process = function(e){//fires evry 0.1sec
	 var inputLe = e.inputBuffer.getChannelData(0); //osc
	 var inputRe = e.inputBuffer.getChannelData(1); //audio bereits verschnellert;
	 var demodulatedSample = 0.0;
	 for(var i=0;i<inputRe.length;i++){
	 	demodulatedSample = inputRe[i]*inputLe[i];
	 	this.demodulatedBuffer.push(this.lowShelfFilter.singleStep(this.firLpf.singleStep(demodulatedSample)));
	 }
};

Decoder.prototype.finish = function(){
	this.demodulator.disconnect();
	this.demodulator.onaudioprocess = null;
	this.demodulator = null;
	this.osc.stop();
	this.osc = null;
	this.notchFilter = null;
	this.merger = null;
	var encoder = new WavAudioEncoder(this.Fs, 2);//WavEncoder is global in document;
	encoder.encode([this.demodulatedBuffer,this.demodulatedBuffer]);//must bei inside array[]
	var blobWav = encoder.finish();
	this.demodulatedBuffer = [];
	this.returnBlobWav(blobWav);
	clearInterval(this.inter);
	this.decodeProcessEnd();
};

Decoder.prototype.demodulateBuffer = function(buffer){
	var l = buffer.length;
	var data;
	while(l--){
		if(buffer[l] !== 0) break;
	}
	data = buffer.subarray(0, l+5);
	if(1/this.Fs*data.length > 10){//min record time 1sec * ten times longer 10
	this.init(this.audioCtx.createBufferSource(), data);
	this.endTime = (1/this.Fs*data.length)/10+this.audioCtx.currentTime;
	this.inter = setInterval(this.decodeProcess,0);
	}else{
		this.demodError("RX duration musst be longer then 10 seconds")
	}
};